(function() {
  var Q, debug, now, parseBody, parseResBody, qs;

  qs = require('querystring');

  Q = require('q');

  debug = require('debug')('request-logger');

  now = function() {
    return (new Date()).getTime();
  };

  parseBody = function(req) {
    var body, error, _base;
    if (req.body) {
      body = (typeof (_base = req.body).toString === "function" ? _base.toString() : void 0) || req.body;
      if (req._json) {
        try {
          body = JSON.parse(body);
        } catch (_error) {
          error = _error;
        }
      } else {
        try {
          body = qs.parse(body);
        } catch (_error) {
          error = _error;
        }
      }
      return body;
    }
  };

  parseResBody = function(buffer, bodyLen, encoding) {
    var body, i;
    if (buffer.length && Buffer.isBuffer(buffer[0])) {
      body = new Buffer(bodyLen);
      i = 0;
      buffer.forEach(function(chunk) {
        chunk.copy(body, i, 0, chunk.length);
        return i += chunk.length;
      });
      return body.toString();
    } else if (buffer.length) {
      if (self.encoding === 'utf8' && buffer[0].length > 0 && buffer[0][0] === '﻿\uFEFF') {
        buffer[0] = buffer[0].substring(1);
      }
      return buffer.join('');
    }
  };

  module.exports = function(req, requestStart, requestEnd) {
    var bodyLen, buffer, endLog, model, promise, res, start, startLog, _ref;
    _ref = [], res = _ref[0], start = _ref[1], model = _ref[2], promise = _ref[3], bodyLen = _ref[4];
    buffer = [];
    bodyLen = 0;
    startLog = function() {
      var data, defer;
      debug('Start log');
      defer = Q.defer();
      data = {
        url: req.href,
        method: req.method,
        headers: req.headers,
        querystring: qs.parse(req.uri.query),
        body: parseBody(req),
        start: new Date()
      };
      start = now();
      requestStart(data, function(err) {
        if (err) {
          return defer.reject(err);
        } else {
          return defer.resolve(model);
        }
      });
      return promise = defer.promise;
    };
    endLog = function() {
      var body, data, error, json, largeBodyLimit;
      debug('End log');
      largeBodyLimit = 40000;
      body = parseResBody(buffer, bodyLen);
      if (body.length > largeBodyLimit) {
        json = {
          error: 'JSON body too large.'
        };
      } else {
        try {
          json = JSON.parse(body);
        } catch (_error) {
          error = _error;
        }
      }
      data = {
        statusCode: res.statusCode,
        resHeaders: res.headers || {},
        resJSON: json,
        resBody: body != null ? typeof body.substr === "function" ? body.substr(0, largeBodyLimit) : void 0 : void 0,
        end: new Date(),
        time: now() - start
      };
      return requestEnd(data);
    };
    req.once('request', function() {
      startLog();
      req.once('response', function(_res) {
        res = _res;
        return res.on('data', function(chunk) {
          buffer.push(chunk);
          return bodyLen += chunk.length;
        });
      });
      return req.once('end', function() {
        return promise.then(function() {
          return endLog();
        });
      });
    });
    return req;
  };

}).call(this);
