(function() {
  var Q, cloneDeep, hideDataFromKeys, logger, objectAssign, traverse;

  Q = require('q');

  traverse = require('traverse');

  cloneDeep = require('lodash.clonedeep');

  logger = require('./request-logger');

  objectAssign = require('object-assign');

  module.exports = function(tableName, sequelize, options) {
    var Model, log, logError;
    Model = require('./model')(tableName, sequelize, sequelize.constructor);
    options = objectAssign({
      hideKeys: [],
      hideValue: '******'
    }, options);
    logError = function(err) {
      return console.error(err);
    };
    log = function(req) {
      var end, model, start;
      model = null;
      start = function(data, cb) {
        data.resHeaders = {};
        data.resJSON = {};
        if (options.hideKeys.length > 0) {
          data = hideDataFromKeys(data, options.hideKeys, options.hideValue);
        }
        model = Model.build(data);
        return Q.when(model.save())["catch"](logError).then(function() {
          return cb();
        });
      };
      end = function(data) {
        if (options.hideKeys.length > 0) {
          data = hideDataFromKeys(data, options.hideKeys, options.hideValue);
        }
        model.setAttributes(data);
        return Q.when(model.save())["catch"](logError).then(function() {
          return req.emit('logger-end', model);
        });
      };
      return logger(req, start, end);
    };
    log.Model = Model;
    return log;
  };

  hideDataFromKeys = function(obj, hideKeys, hideValue) {
    obj = cloneDeep(obj);
    traverse(obj).forEach(function(data) {
      if (~hideKeys.indexOf(this.key)) {
        return this.update(hideValue);
      }
    });
    return obj;
  };

}).call(this);
